require("xml_to_table")

local ccallback,pcallback

local function XMLDateToDate(str)
	if not str then 
		return
		end
	local date={}
	if string.len(str)<14 then 
		return
		end
	local tz
	local a,b
	a,b,date.Year,date.Month,date.Day,date.Hour,date.Minute,date.Second,tz=string.find(str,"%s*(%d%d%d%d)(%d%d)(%d%d)(%d%d)(%d%d)(%d%d)%s*([%+%-]%d%d%d%d)%s*")
	if a==nil then
		--try match without timezone
		a,b,date.Year,date.Month,date.Day,date.Hour,date.Minute,date.Second,tz=string.find(str,"%s*(%d%d%d%d)(%d%d)(%d%d)(%d%d)(%d%d)(%d%d)%s*")
		end
	if a==nil then print(string.format("Invalid date in xmltv: \"%s\"",str)) end
	if tz and(tz~="") then
		local tzs,tzh,tzm
		_,_,tzs,tzh,tzm=string.find(tz,"%s*([%+%-])(%d%d)(%d%d)%s*")
		if tzs=="+" then
			date.Hour=date.Hour-tzh
			date.Minute=date.Minute-tzm
			elseif tzs=="-" then
				date.Hour=date.Hour+tzh
				date.Minute=date.Minute+tzm
				end
		date=time.Fix(date)
		end
	return date
	end

local function LangStrToXMLStr(tag)
	if not tag then
		return
		end
	local e
	for _,e in tag do
		local str={}
		if e.Attrs.lang then
			str.Lang=e.Attrs.lang
			end
		str.Value=e.Data
		return str
		end
	end
	
local function LangStrToXMLStrList(tag)
	if not tag then
		return
		end
	local e
	local res={}
	for _,e in tag do
		local str={}
		if e.Attrs.lang then
			str.Lang=e.Attrs.lang
			end
		str.Value=e.Data
		table.insert(res,str)
		end
	return res
	end

local function GetIcon(tag)
	if not tag then
		return
		end
	local e
	local res={}
	for _,e in tag do
		local ico={}
		ico.Source=e.Attrs.src
		ico.Width=e.Attrs.width
		ico.Height=e.Attrs.height
		table.insert(res,ico)
		end
	return res
	end
	
local function GetStrList(tag)
	if not tag then
		return
		end
	local e
	local res={}
	for _,e in tag do
		table.insert(res,e.Data)
		end
	return res
	end
	
local function GetCredits(tag)
	if not tag then
		return
		end
	local res={}
	res.Director=GetStrList(tag[1].Children.director)
	res.Actor=GetStrList(tag[1].Children.actor)
	res.Writer=GetStrList(tag[1].Children.writer)
	res.Adapter=GetStrList(tag[1].Children.adapter)
	res.Producer=GetStrList(tag[1].Children.producer)
	res.Presenter=GetStrList(tag[1].Children.presenter)
	res.Commentator=GetStrList(tag[1].Children.commentator)
	res.Guest=GetStrList(tag[1].Children.guest)
	return res
	end

local function GetLength(tag)
	if not tag then
		return
		end
	local res={}
	res.Length=tag[1].Data
	local s=tag[1].Attrs.units
	if s=="seconds" then res.Units=XGUI.LengthUnits.Seconds
		elseif s=="minutes" then res.Units=XGUI.LengthUnits.Minutes
		elseif s=="hours" then res.Units=XGUI.LengthUnits.Hours
		end
	return res
	end
	
local function GetEpisode(tag)
	if not tag then
		return
		end
	local res={}
	res.Number=tag[1].Data
	res.System=tag[1].Attrs.system
	return res
	end
	
local function GetVideo(tag)
	if not tag then
		return
		end
	local res={}
	res.Present=(tag[1].Children.present==nil)or(tag[1].Children.present[1].Data~="no")
	res.Colour=(tag[1].Children.colour==nil)or(tag[1].Children.colour[1].Data~="no")
	if tag[1].Children.aspect then
		res.Aspect=tag[1].Children.aspect[1].Data
		end
	if tag[1].Children.quality then
		res.quality=tag[1].Children.quality[1].Data
		end
	return res
	end
	
local function GetAudio(tag)
	if not tag then
		return
		end
	local res={}
	res.Present=(tag[1].Children.present==nil)or(tag[1].Children.present[1].Data~="no")
	if tag[1].Children.stereo then
		res.Stereo=tag[1].Children.stereo[1].Data
		end
	return res
	end
	
local function GetPrevious(tag)
	if not tag then
		return
		end
	local res={}
	res.Channel=tag[1].Attrs.channel
	res.Start=tag[1].Attrs.start
	return res
	end
	
local function GetSubtitles(tag)
	if not tag then
		return
		end
	local res={}
	for _,e in tag do
		local sub={}
		local s=e.Attrs.type
		if s=="teletext" then
			sub.Type=XGUI.SubtitlesType.Teletext
			else sub.Type=XGUI.SubtitlesType.OnScreen
			end
		sub.Lang=LangStrToXMLStr(e.Children.language)
		table.insert(res,sub)
		end
	return res
	end
	
local function GetRating(tag)
	if not tag then
		return
		end
	local res={}
	for _,e in tag do
		local rat={}
		rat.Value=e.Children.value[1].Data
		rat.Icon=GetIcon(e.Children.icon)
		rat.System=e.Attrs.system
		table.insert(res,rat)
		end
	return res
	end
	
local function GetStarRating(tag)
	if not tag then
		return
		end
	local res={}
	if tag[1].Children.value then
		_,_,res.Number,res.Outof=string.find(tag[1].Children.value[1].Data,"%s*(%d*)%s*/%s*(%d*)%s*")
		end
	res.Icon=GetIcon(tag[1].Children.icon)
	return res	
	end
	
local function TagCallback(name,tag)
	if name=="programme" then
		local prog={}
		prog.Channel=tag.Attrs.channel
		prog.Start=XMLDateToDate(tag.Attrs.start)
		prog.Stop=XMLDateToDate(tag.Attrs.stop)
		prog.PDCStart=XMLDateToDate(tag.Attrs["pdc-start"])
		prog.VPSStart=XMLDateToDate(tag.Attrs["vps-start"])
		prog.ShowView=tag.Attrs.showview
		prog.VideoPlus=tag.Attrs.videoplus
		prog.Clump=tag.Attrs.clumpidx
		prog.Title=LangStrToXMLStrList(tag.Children.title)
		prog.Sub_Title=LangStrToXMLStrList(tag.Children["sub-title"])
		prog.Description=LangStrToXMLStrList(tag.Children.desc)
		prog.Credits=GetCredits(tag.Children.credits)
		prog.Date=tag.Children.date
		prog.Category=LangStrToXMLStrList(tag.Children.category)
		prog.Lang=LangStrToXMLStr(tag.Children.language)		
		prog.OriginalLang=LangStrToXMLStr(tag.Children["orig-language"])		
		prog.Length=GetLength(tag.Children.length)
		prog.Icon=GetIcon(tag.Children.Icon)
		prog.URL=GetStrList(tag.Children.url)
		prog.Country=LangStrToXMLStrList(tag.Children.country)
		prog.Episode=GetEpisode(tag.Children["episode-num"])
		prog.Video=GetVideo(tag.Children.video)
		prog.Audio=GetAudio(tag.Children.audio)
		prog.PreviouslyShown=GetPrevious(tag.Children["previously-shown"])
		prog.Premiere=LangStrToXMLStr(tag.Children.premiere)
		prog.LastChance=LangStrToXMLStr(tag.Children["last-chance"])
		prog.New= (tag.Children.new~=nil)
		prog.Subtitles=GetSubtitles(tag.Children.subtitles)
		prog.Rating=GetRating(tag.Children.rating)
		prog.StarRating=GetStarRating(tag.Children["star-rating"])
		if pcallback then
			local e,msg=pcall(pcallback,prog)
			if not e then
				print("Error calling program callback: "..msg)
				end
			end
		elseif name=="channel" then
			local chan={}
			chan.ID=tag.Attrs.id
			chan.Name=LangStrToXMLStrList(tag.Children["display-name"])
			chan.Icon=GetIcon(tag.Children.icon)
			chan.URL=GetStrList(tag.Children.url)
			if ccallback then
				local e,msg=pcall(ccallback,chan)
				if not e then
					print("Error calling channel callback: "..msg)
					end
				end
			end
	end

XmltvParse={
	ParseFile = function(file,ProgramCallback,ChannelCallback)
		pcallback=ProgramCallback
		ccallback=ChannelCallback
		XmlToTable.ParseFile(file,1,TagCallback)
		end,
	
	ParseString = function(str,ProgramCallback,ChannelCallback)
		pcallback=ProgramCallback
		ccallback=ChannelCallback
		XmlToTable.ParseString(str,1,TagCallback)
		end,
	}
	